"""
Salt Module to perform CRUD operations for NSX-T uplink profiles
"""
import logging

from saltext.vmware.utils import common
from saltext.vmware.utils import nsxt_request

log = logging.getLogger(__name__)

__virtual_name__ = "nsxt_uplink_profiles"


def __virtual__():
    return __virtual_name__


UPLINK_PROFILES_BASE_URL = "https://{0}/api/v1/host-switch-profiles"

create_params_for_uplink_profies = [
    "lags",
    "mtu",
    "named_teamings",
    "overlay_encap",
    "required_capabilities",
    "tags",
    "transport_vlan",
    "description",
]


def get(
    hostname,
    username,
    password,
    verify_ssl=True,
    cert=None,
    cert_common_name=None,
    cursor=None,
    deployment_type=None,
    hostswitch_profile_type=None,
    include_system_owned=None,
    included_fields=None,
    node_type=None,
    page_size=None,
    sort_ascending=None,
    sort_by=None,
    uplink_teaming_policy_name=None,
):
    """
    Lists NSX-T up-link profiles

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_uplink_profiles.get hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification.
        If the client certificate common name and hostname do not match (in case of self-signed certificates),
        specify the certificate common name as part of this parameter. This value is then used to compare against
        certificate common name.

    cursor
        (Optional) Opaque cursor to be used for getting next page of records (supplied by current result page)

    deployment_type
        (Optional) Deployment type of EdgeNode or PublicCloudGatewayNode
        If the node_type is specified, then deployment_type may be specified to filter uplink profiles applicable to
        only PHYSICAL_MACHINE or VIRTUAL_MACHINE deployments of these nodes.

    hostswitch_profile_type
        (Optional) Type of host switch profile

    include_system_owned
        (Optional) Boolean. Whether the list result contains system resources

    included_fields
        (Optional) Comma separated list of fields that should be included in query result

    node_type
        (Optional) Fabric node type for which uplink profiles are to be listed. The fabric node type is the resource_type of the
        Node such as EdgeNode and PublicCloudGatewayNode. If a fabric node type is given, uplink profiles that apply
        for nodes of the given type will be returned.

    page_size
        (Optional) Maximum number of results to return in this page

    sort_ascending
        (Optional) Boolean

    sort_by
        (Optional) Field by which records are sorted

    uplink_teaming_policy_name
        (Optional) The host switch profile's uplink teaming policy name. If populated, only UplinkHostSwitchProfiles with the
        specified uplink teaming policy name are returned. Otherwise, any HostSwitchProfile can be returned.
    """
    log.info("Fetching NSX-T uplink profiles")
    url = UPLINK_PROFILES_BASE_URL.format(hostname)
    params = common._filter_kwargs(
        allowed_kwargs=[
            "cursor",
            "deployment_type",
            "hostswitch_profile_type",
            "include_system_owned",
            "included_fields",
            "node_type",
            "page_size",
            "sort_ascending",
            "sort_by",
            "uplink_teaming_policy_name",
        ],
        default_dict=None,
        cursor=cursor,
        deployment_type=deployment_type,
        hostswitch_profile_type=hostswitch_profile_type,
        include_system_owned=include_system_owned,
        included_fields=included_fields,
        page_size=page_size,
        sort_ascending=sort_ascending,
        sort_by=sort_by,
        uplink_teaming_policy_name=uplink_teaming_policy_name,
        node_type=node_type,
    )
    return nsxt_request.call_api(
        method="get",
        url=url,
        username=username,
        password=password,
        cert_common_name=cert_common_name,
        verify_ssl=verify_ssl,
        cert=cert,
        params=params,
    )


def get_by_display_name(
    hostname,
    username,
    password,
    display_name,
    verify_ssl=True,
    cert=None,
    cert_common_name=None,
):
    """
    Gets nsxt uplink profiles(UplinkHostSwitchProfile) present in the NSX-T Manager with given display_name.

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_uplink_profiles.get_by_display_name hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    display_name
        The name of the uplink profile

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against certificate common name
    """
    log.info("Finding uplink profiles with display name: %s", display_name)
    uplink_profiles = common._read_paginated(
        func=get,
        display_name=display_name,
        hostname=hostname,
        username=username,
        password=password,
        verify_ssl=verify_ssl,
        cert=cert,
        cert_common_name=cert_common_name,
    )
    if "error" in uplink_profiles:
        return uplink_profiles
    return {"results": uplink_profiles}


def create(
    hostname,
    username,
    password,
    display_name,
    teaming,
    verify_ssl=True,
    cert=None,
    cert_common_name=None,
    lags=None,
    mtu=None,
    named_teamings=None,
    overlay_encap=None,
    required_capabilities=None,
    tags=None,
    transport_vlan=None,
    description=None,
):
    """
    Creates uplink profile(resource_type: UplinkHostSwitchProfile)

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_uplink_profiles.create hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    display_name
        The name of the uplink profile

    teaming
        Default TeamingPolicy associated with this UplinkProfile. Object with following parameters:
        Example:

        .. code::

            {'standby_list':[],'active_list':[{'uplink_name':'uplink3','uplink_type':'PNIC'}],'policy':'FAILOVER_ORDER'}

        active_list
            List of Uplinks used in active list. Array of Uplink objects.

            .. code::

                 active_list='[
                    {
                        "uplink_name": "uplink3",
                        "uplink_type": "PNIC"
                    }
                ]'

            Parameters as follows:

            uplink_name
                Name of this uplink

            uplink_type
                Type of the uplink. PNIC or LAG

        policy
            Teaming policy. Required field.
            Values could be one among FAILOVER_ORDER, LOADBALANCE_SRCID, LOADBALANCE_SRC_MAC

        standby_list
            List of Uplinks used in standby list. Array of Uplink objects.

            .. code::

                standby_list='[
                    {
                        "uplink_name": "uplink2",
                        "uplink_type": "PNIC"
                    }
                ]'

            Parameters as follows:

            uplink_name
                Name of this uplink

            uplink_type
                Type of the uplink. PNIC or LAG

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against certificate common name

    lags
        (Optional) list of LACP group

    mtu
        (Optional) Maximum Transmission Unit used for uplinks

    named_teamings
        (Optional) List of named uplink teaming policies that can be used by logical switches.
         Array of NamedTeamingPolicy

    overlay_encap
        (Optional) The protocol used to encapsulate overlay traffic

    required_capabilities
        (Optional) List of string

    tags
        (Optional) Opaque identifier meaninful to API user. Array of Tag

    transport_vlan
        (Optional) VLAN used for tagging Overlay traffic of associated HostSwitch. Type: integer

    description
        (Optional) Description for the resource
    """
    log.info("Creating nsxt uplink profile")
    url = UPLINK_PROFILES_BASE_URL.format(hostname)
    req_data = common._filter_kwargs(
        allowed_kwargs=create_params_for_uplink_profies,
        default_dict={},
        lags=lags,
        mtu=mtu,
        named_teamings=named_teamings,
        overlay_encap=overlay_encap,
        required_capabilities=required_capabilities,
        tags=tags,
        transport_vlan=transport_vlan,
        description=description,
    )
    req_data["display_name"] = display_name
    req_data["resource_type"] = "UplinkHostSwitchProfile"
    req_data["teaming"] = teaming
    return nsxt_request.call_api(
        method="post",
        url=url,
        username=username,
        password=password,
        cert_common_name=cert_common_name,
        verify_ssl=verify_ssl,
        cert=cert,
        data=req_data,
    )


def update(
    hostname,
    username,
    password,
    display_name,
    teaming,
    uplink_profile_id,
    revision,
    verify_ssl=True,
    cert=None,
    cert_common_name=None,
    lags=None,
    mtu=None,
    named_teamings=None,
    overlay_encap=None,
    required_capabilities=None,
    tags=None,
    transport_vlan=None,
    description=None,
):
    """
    Updates uplink profile(resource_type: UplinkHostSwitchProfile)

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_uplink_profiles.update hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    display_name
        The name of the uplink profile

    teaming
        Default TeamingPolicy associated with this UplinkProfile. Object with following parameters:
        Example:

        .. code::

            {'standby_list':[],'active_list':[{'uplink_name':'uplink3','uplink_type':'PNIC'}],'policy':'FAILOVER_ORDER'}

        active_list
            List of Uplinks used in active list. Array of Uplink objects.

            .. code::

                active_list='[
                    {
                        "uplink_name": "uplink3",
                        "uplink_type": "PNIC"
                    }
                ]'

            Parameters as follows:

            uplink_name
                Name of this uplink

            uplink_type
                Type of the uplink. PNIC or LAG

        policy
            Teaming policy. Required field.
            Values could be one among FAILOVER_ORDER, LOADBALANCE_SRCID, LOADBALANCE_SRC_MAC

        standby_list
            List of Uplinks used in standby list. Array of Uplink objects.

            .. code::

                standby_list='[
                    {
                        "uplink_name": "uplink2",
                        "uplink_type": "PNIC"
                    }
                ]'

            Parameters as follows:

            uplink_name
                Name of this uplink

            uplink_type
                Type of the uplink. PNIC or LAG

    uplink_profile_id
        Unique id provided by NSX-T for uplink profile

    revision
        _revision property of the uplink profile provided by NSX-T

    verify_ssl
        Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against certificate common name

    lags
        (Optional) list of LACP group

    mtu
        (Optional) Maximum Transmission Unit used for uplinks

    named_teamings
        (Optional) List of named uplink teaming policies that can be used by logical switches. Array of NamedTeamingPolicy

    overlay_encap
        (Optional) The protocol used to encapsulate overlay traffic

    required_capabilities
        (Optional) List of string

    tags
        (Optional) Opaque identifier meaninful to API user. Array of Tag

    transport_vlan
        (Optional) VLAN used for tagging Overlay traffic of associated HostSwitch. Type: integer

    description
        (Optional) Description for the resource
    """
    log.info("Updating nsxt uplink profile %s", display_name)
    url = UPLINK_PROFILES_BASE_URL.format(hostname) + "/{}".format(uplink_profile_id)
    req_data = common._filter_kwargs(
        allowed_kwargs=create_params_for_uplink_profies,
        default_dict={},
        lags=lags,
        mtu=mtu,
        named_teamings=named_teamings,
        overlay_encap=overlay_encap,
        required_capabilities=required_capabilities,
        tags=tags,
        transport_vlan=transport_vlan,
        description=description,
    )
    req_data["display_name"] = display_name
    req_data["resource_type"] = "UplinkHostSwitchProfile"
    req_data["teaming"] = teaming
    req_data["_revision"] = revision
    return nsxt_request.call_api(
        method="put",
        url=url,
        username=username,
        password=password,
        cert_common_name=cert_common_name,
        verify_ssl=verify_ssl,
        cert=cert,
        data=req_data,
    )


def delete(
    hostname,
    username,
    password,
    uplink_profile_id,
    verify_ssl=True,
    cert=None,
    cert_common_name=None,
):
    """
    Deletes uplink profile(UplinkHostSwitchProfile)

    CLI Example:

    .. code-block:: bash

        salt vm_minion nsxt_uplink_profiles.delete hostname=nsxt-manager.local username=admin ...

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    uplink_profile_id
        Existing uplink profile id

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against certificate common name
    """
    log.info("Deleting uplink profile with id %s", uplink_profile_id)
    url = UPLINK_PROFILES_BASE_URL.format(hostname) + "/{}".format(uplink_profile_id)
    result = nsxt_request.call_api(
        method="delete",
        url=url,
        username=username,
        password=password,
        cert_common_name=cert_common_name,
        verify_ssl=verify_ssl,
        cert=cert,
    )
    return result or {"message": "Deleted uplink profile successfully"}



"""
Salt execution module for nat rules
Provides methods to Create, Update, Read and Delete nat rules.
"""
import logging
import os

from saltext.vmware.utils import vmc_constants
from saltext.vmware.utils import vmc_request
from saltext.vmware.utils import vmc_templates

log = logging.getLogger(__name__)

__virtualname__ = "vmc_nat_rules"


def __virtual__():
    return __virtualname__


def _create_payload_for_nat_rule(rule_id, user_input):
    """
    This function creates the payload based on the template and user input passed
    """
    data = vmc_request.create_payload_for_request(vmc_templates.create_nat_rules, user_input)
    data["id"] = data["display_name"] = rule_id
    return data


def get(
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    tier1,
    nat,
    verify_ssl=True,
    cert=None,
    cursor=None,
    page_size=None,
    sort_by=None,
    sort_ascending=None,
):
    """
    Retrieves nat rules for Given SDDC

    CLI Example:

    .. code-block:: bash

        salt vm_minion vmc_nat_rules.get hostname=nsxt-manager.local domain_id=mgw ...


    hostname
        The host name of NSX-T manager

    refresh_key
        refresh_key to get access token

    authorization_host
        hostname to get access token

    org_id
        org_id of the SDDC

    sddc_id
        sddc_id for which nat rules should be retrieved

    tier1
        tier1 option are cgw and user defined tier1

    nat
        nat option are USER

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cursor
        (Optional) Opaque cursor to be used for getting next page of records (supplied by current result page)

    page_size
        (Optional) Maximum number of results to return in this page. Default page size is 1000.

    sort_by
        (Optional) Field by which records are sorted

    sort_ascending
        (Optional) Boolean value to sort result in ascending order. Enabled by default.

    """

    log.info("Retrieving nat rules for SDDC %s", sddc_id)

    api_url_base = vmc_request.set_base_url(hostname)
    api_url = (
        "{base_url}vmc/reverse-proxy/api/orgs/{org_id}/sddcs/{sddc_id}/"
        "policy/api/v1/infra/tier-1s/{tier1}/nat/{nat}/nat-rules"
    )
    api_url = api_url.format(
        base_url=api_url_base, org_id=org_id, sddc_id=sddc_id, tier1=tier1, nat=nat
    )

    params = vmc_request._filter_kwargs(
        allowed_kwargs=["cursor", "page_size", "sort_ascending", "sort_by"],
        cursor=cursor,
        page_size=page_size,
        sort_by=sort_by,
        sort_ascending=sort_ascending,
    )

    return vmc_request.call_api(
        method=vmc_constants.GET_REQUEST_METHOD,
        url=api_url,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        description="vmc_nat_rule.get",
        verify_ssl=verify_ssl,
        cert=cert,
        params=params,
    )


def get_by_id(
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    tier1,
    nat,
    nat_rule,
    verify_ssl=True,
    cert=None,
):
    """
    Retrieves specific nat rule for Given SDDC

    CLI Example:

    .. code-block:: bash

        salt vm_minion vmc_nat_rules.get_by_id hostname=nsxt-manager.local tier1=cgw ...

    hostname
        The host name of NSX-T manager

    refresh_key
        refresh_key to get access token

    authorization_host
        hostname to get access token

    org_id
        org_id of the SDDC

    sddc_id
        sddc_id for which nat rules should be retrieved

    tier1
        tier1 option are cgw and user defined tier1

    nat
        nat option are USER/default/Internal

    nat_rule
        id of specific nat rule

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.
    """
    log.info("Retrieving nat rule %s for SDDC %s", nat_rule, sddc_id)
    api_url_base = vmc_request.set_base_url(hostname)
    api_url = (
        "{base_url}vmc/reverse-proxy/api/orgs/{org_id}/sddcs/{sddc_id}/"
        "policy/api/v1/infra/tier-1s/{tier1}/nat/{nat}/nat-rules/{nat_rule}"
    )
    api_url = api_url.format(
        base_url=api_url_base,
        org_id=org_id,
        sddc_id=sddc_id,
        tier1=tier1,
        nat=nat,
        nat_rule=nat_rule,
    )

    return vmc_request.call_api(
        method=vmc_constants.GET_REQUEST_METHOD,
        url=api_url,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        description="vmc_nat_rule.get_by_id",
        verify_ssl=verify_ssl,
        cert=cert,
    )


def delete(
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    tier1,
    nat,
    nat_rule,
    verify_ssl=True,
    cert=None,
):
    """
    Delete nat rules for Given SDDC

    CLI Example:

    .. code-block:: bash

        salt vm_minion vmc_nat_rules.delete hostname=nsxt-manager.local tier1=cgw ...

    hostname
        The host name of NSX-T manager

    refresh_key
        refresh_key to get access token

    authorization_host
        hostname to get access token

    org_id
        org_id of the SDDC

    sddc_id
        sddc_id for which nat rules should be deleted

    tier1
        tier1 option are cgw and user defined tier1

    nat
        nat option are USER/default/Internal

    nat_rule
        id of specific nat rule

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    """
    log.info("Deleting nat rule %s for SDDC %s", nat_rule, sddc_id)
    api_url_base = vmc_request.set_base_url(hostname)
    api_url = (
        "{base_url}vmc/reverse-proxy/api/orgs/{org_id}/sddcs/{sddc_id}/"
        "policy/api/v1/infra/tier-1s/{tier1}/nat/{nat}/nat-rules/{nat_rule}"
    )
    api_url = api_url.format(
        base_url=api_url_base,
        org_id=org_id,
        sddc_id=sddc_id,
        tier1=tier1,
        nat=nat,
        nat_rule=nat_rule,
    )
    return vmc_request.call_api(
        method=vmc_constants.DELETE_REQUEST_METHOD,
        url=api_url,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        description="vmc_nat_rule.delete",
        responsebody_applicable=False,
        verify_ssl=verify_ssl,
        cert=cert,
    )


def create(
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    tier1,
    nat,
    nat_rule,
    verify_ssl=True,
    cert=None,
    action=None,
    destination_network=None,
    source_network=None,
    translated_network=None,
    translated_ports=vmc_constants.VMC_NONE,
    scope=None,
    service=None,
    enabled=None,
    firewall_match=None,
    logging=None,
    description=None,
    tags=vmc_constants.VMC_NONE,
    sequence_number=None,
):
    """
    Create nat rules for Given SDDC

    CLI Example:

    .. code-block:: bash

        salt vm_minion vmc_nat_rules.create hostname=nsxt-manager.local tier1=cgw ...

    hostname
        The host name of NSX-T manager

    refresh_key
        refresh_key to get access token

    authorization_host
        hostname to get access token

    org_id
        org_id of the SDDC

    sddc_id
        sddc_id for which nat rules should be created

    tier1
        tier1 option are cgw and user defined tier1

    nat
        nat option are USER/default/Internal

    nat_rule
        id of specific nat rule

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    action
        specify type of nat rule it can have value REFLEXIVE, DNAT

            REFLEXIVE nat rule require
                source_network
                translated_network
                service should be empty
                translated_ports  should be None
                destination_network should be none

            DNAT  Rule require
                destination_network
                translated_network
                translated_ports can be none
                service can be none
                source_network can be None or input network.

    destination_network
        Represents the destination network
            This supports single IP address or comma separated list of single IP
            addresses or CIDR. This does not support IP range or IP sets.

    source_network
        Represents the source network address
            This supports single IP address or comma separated list of single IP
            addresses or CIDR. This does not support IP range or IP sets.

    translated_network
        Represents the translated network address

            This supports single IP address or comma separated list of single IP
            addresses or CIDR. This does not support IP range or IP sets.

    translated_ports
        Port number or port range

            Please note, if there is service configured in this nat rule, the translated_port
            will be realized on NSX Manager as the destination_port. If there is no sevice configured,
            the port will be ignored.

    scope
        (Optional) Array of policy paths of labels, ProviderInterface, NetworkInterface
        If this value is not passed, then ["/infra/labels/cgw-public"] will be used by default.

    service
        (Optional) Represents the service on which the nat rule will be applied
        If this value is not passed, then empty string will be used by default.

    enabled
        (Optional) Policy nat rule enabled flag

            The flag, which suggests whether the nat rule is enabled or
            disabled. The default is True.

    firewall_match
        (Optional) Represents the firewall match flag

            It indicates how the firewall matches the address after nating if firewall
            stage is not skipped.
            possible values: MATCH_EXTERNAL_ADDRESS, MATCH_INTERNAL_ADDRESS
            Default: "MATCH_INTERNAL_ADDRESS"

    logging
        (Optional) Policy nat rule logging flag
            default: False

    description
        (Optional) Description of nat rule

    tags
        (Optional) Opaque identifiers meaningful to the API user. Maximum 30 tags can be associated:

        .. code-block::

            tags='[
                {
                    "tag": "<tag-key-1>"
                    "scope": "<tag-value-1>"
                },
                {
                    "tag": "<tag-key-2>"
                    "scope": "<tag-value-2>"
                }
            ]'


    sequence_number
        (Optional) Sequence number of the nat rule
            The sequence_number decides the rule_priority of a nat rule.
            default: 0
            type: int

    Example values:

        .. code-block::

            {
                "action": "REFLEXIVE",
                "translated_network": "203.0.113.36",
                "translated_ports": null,
                "destination_network": "",
                "source_network": "198.51.100.23",
                "sequence_number": 0,
                "service": "",
                "logging": false,
                "enabled": false,
                "scope": [
                    "/infra/labels/cgw-public"
                ],
                "tags": [
                    {
                        "tag": "tag1",
                        "scope": "scope1"
                    }
                ],
                "description": "",
                "firewall_match": "MATCH_INTERNAL_ADDRESS"
            }

    Please refer the `Nat Rule <https://developer.vmware.com/docs/nsx-vmc-policy/latest/data-structures/InlinePolicyNatRule1/>`_ to get insight of input parameters.

    """
    log.info("Creating nat rule %s for SDDC %s ", nat_rule, sddc_id)
    api_url_base = vmc_request.set_base_url(hostname)
    api_url = (
        "{base_url}vmc/reverse-proxy/api/orgs/{org_id}/sddcs/{sddc_id}/"
        "policy/api/v1/infra/tier-1s/{tier1}/nat/{nat}/nat-rules/{nat_rule}"
    )
    api_url = api_url.format(
        base_url=api_url_base,
        org_id=org_id,
        sddc_id=sddc_id,
        tier1=tier1,
        nat=nat,
        nat_rule=nat_rule,
    )

    allowed_dict = {
        "action": action,
        "description": description,
        "destination_network": destination_network,
        "scope": scope,
        "service": service,
        "source_network": source_network,
        "tags": tags,
        "translated_network": translated_network,
        "translated_ports": translated_ports,
        "enabled": enabled,
        "firewall_match": firewall_match,
        "logging": logging,
        "sequence_number": sequence_number,
    }
    req_data = vmc_request._filter_kwargs(
        allowed_kwargs=allowed_dict.keys(), allow_none=["translated_ports", "tags"], **allowed_dict
    )

    data = _create_payload_for_nat_rule(nat_rule, req_data)

    return vmc_request.call_api(
        method=vmc_constants.PUT_REQUEST_METHOD,
        url=api_url,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        description="vmc_nat_rule.create",
        data=data,
        verify_ssl=verify_ssl,
        cert=cert,
    )


def update(
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    sddc_id,
    tier1,
    nat,
    nat_rule,
    verify_ssl=True,
    cert=None,
    action=None,
    destination_network=None,
    source_network=None,
    translated_network=None,
    translated_ports=vmc_constants.VMC_NONE,
    scope=None,
    service=None,
    enabled=None,
    firewall_match=None,
    logging=None,
    description=None,
    tags=vmc_constants.VMC_NONE,
    sequence_number=None,
    display_name=None,
):
    """
    Update nat rule for Given SDDC

    CLI Example:

    .. code-block:: bash

        salt vm_minion vmc_nat_rules.update hostname=nsxt-manager.local tier1=cgw ...


    hostname
        The host name of NSX-T manager

    refresh_key
        refresh_key to get access token

    authorization_host
        hostname to get access token

    org_id
        org_id of the SDDC

    sddc_id
        sddc_id for which nat rules should be updated

    tier1
        tier1 option are cgw and user defined tier1

    nat
        nat option are USER/default/Internal

    nat_rule
        id of specific nat rule

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    action
        specify type of nat rule it can have value REFLEXIVE, DNAT

            REFLEXIVE nat rule require
                source_network
                translated_network
                service should be empty
                translated_ports  should be None
                destination_network should be none

            DNAT  Rule require
                destination_network
                translated_network
                translated_ports can be none
                service can be none
                source_network can be None or input network.

    destination_network
        Represents the destination network
            This supports single IP address or comma separated list of single IP
            addresses or CIDR. This does not support IP range or IP sets.

    source_network
        Represents the source network address
            This supports single IP address or comma separated list of single IP
            addresses or CIDR. This does not support IP range or IP sets.

    translated_network
        Represents the translated network address

            This supports single IP address or comma separated list of single IP
            addresses or CIDR. This does not support IP range or IP sets.

    translated_ports
        Port number or port range

            Please note, if there is service configured in this nat rule, the translated_port
            will be realized on NSX Manager as the destination_port. If there is no sevice configured,
            the port will be ignored.

    scope
        (Optional) Array of policy paths of labels, ProviderInterface, NetworkInterface
        If this value is not passed, then ["/infra/labels/cgw-public"] will be used by default.

    service
        (Optional) Represents the service on which the nat rule will be applied
        If this value is not passed, then empty string will be used by default.

    enabled
        (Optional) Policy nat rule enabled flag

            The flag, which suggests whether the nat rule is enabled or
            disabled. The default is True.

    firewall_match
        (Optional) Represents the firewall match flag

            It indicates how the firewall matches the address after nating if firewall
            stage is not skipped.
            possible values: MATCH_EXTERNAL_ADDRESS, MATCH_INTERNAL_ADDRESS
            Default: "MATCH_INTERNAL_ADDRESS"

    logging
        (Optional) Policy nat rule logging flag
            default: False

    description
        (Optional) Description of nat rule

    tags
        (Optional) Opaque identifiers meaningful to the API user. Maximum 30 tags can be associated:

        .. code-block::

            tags='[
                {
                    "tag": "<tag-key-1>"
                    "scope": "<tag-value-1>"
                },
                {
                    "tag": "<tag-key-2>"
                    "scope": "<tag-value-2>"
                }
            ]'

    sequence_number
        (Optional) Sequence number of the Nat Rule
            The sequence_number decides the rule_priority of a nat rule.
            default: 0
            type: int

    display_name
        Identifier to use when displaying entity in logs or GUI

    Example values:

        .. code-block::

            {
                "action": "REFLEXIVE",
                "translated_network": "203.0.113.36",
                "translated_ports": null,
                "destination_network": "",
                "source_network": "198.51.100.23",
                "sequence_number": 0,
                "service": "",
                "logging": false,
                "enabled": false,
                "scope": [
                    "/infra/labels/cgw-public"
                ],
                "tags": [
                    {
                        "tag": "tag1",
                        "scope": "scope1"
                    }
                ],
                "description": "",
                "firewall_match": "MATCH_INTERNAL_ADDRESS"
            }

    Please refer the `Nat Rule <https://developer.vmware.com/docs/nsx-vmc-policy/latest/data-structures/InlinePolicyNatRule1/>`_ to get insight of input parameters

    """
    log.info("Updating Nat rule %s for SDDC %s ", nat_rule, sddc_id)
    api_url_base = vmc_request.set_base_url(hostname)
    api_url = (
        "{base_url}vmc/reverse-proxy/api/orgs/{org_id}/sddcs/{sddc_id}/"
        "policy/api/v1/infra/tier-1s/{tier1}/nat/{nat}/nat-rules/{nat_rule}"
    )
    api_url = api_url.format(
        base_url=api_url_base,
        org_id=org_id,
        sddc_id=sddc_id,
        tier1=tier1,
        nat=nat,
        nat_rule=nat_rule,
    )

    # fetch the nat rule for the given nat_rule
    existing_data = get_by_id(
        hostname,
        refresh_key,
        authorization_host,
        org_id,
        sddc_id,
        tier1,
        nat,
        nat_rule,
        verify_ssl,
        cert,
    )

    if vmc_constants.ERROR in existing_data:
        return existing_data

    allowed_dict = {
        "action": action,
        "description": description,
        "destination_network": destination_network,
        "scope": scope,
        "service": service,
        "source_network": source_network,
        "tags": tags,
        "translated_network": translated_network,
        "translated_ports": translated_ports,
        "enabled": enabled,
        "firewall_match": firewall_match,
        "logging": logging,
        "sequence_number": sequence_number,
        "display_name": display_name,
    }

    req_data = vmc_request._filter_kwargs(
        allowed_kwargs=allowed_dict.keys(), allow_none=["translated_ports", "tags"], **allowed_dict
    )

    payload = vmc_request.create_payload_for_request(
        vmc_templates.update_nat_rules, req_data, existing_data
    )
    return vmc_request.call_api(
        method=vmc_constants.PATCH_REQUEST_METHOD,
        url=api_url,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        description="vmc_nat_rules.update",
        responsebody_applicable=False,
        data=payload,
        verify_ssl=verify_ssl,
        cert=cert,
    )



# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
# pylint: disable=no-name-in-module
import logging

import salt.exceptions
import salt.modules.cmdmod
import salt.utils.path
import salt.utils.platform
import salt.utils.stringutils
from saltext.vmware.utils.common import create_datacenter as create_datacenter
from saltext.vmware.utils.common import delete_datacenter as delete_datacenter
from saltext.vmware.utils.common import get_datacenter as get_datacenter
from saltext.vmware.utils.common import get_datacenters as get_datacenters
from saltext.vmware.utils.common import list_datacenters as list_datacenters

try:
    from pyVmomi import vim, vmodl

    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False

log = logging.getLogger(__name__)


def get_vm_datacenter(*, vm):
    """
    Return a datacenter from vm
    """
    datacenter = None
    while True:
        if isinstance(vm, vim.Datacenter):
            datacenter = vm
            break
        try:
            vm = vm.parent
        except AttributeError:
            break
    return datacenter



# Copyright 2021 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import logging

import salt.exceptions
import saltext.vmware.utils.cluster as utils_cluster
import saltext.vmware.utils.connect as connect
import saltext.vmware.utils.datacenter as utils_datacenter
import saltext.vmware.utils.esxi as utils_esxi

log = logging.getLogger(__name__)

try:
    from pyVmomi import vim

    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False


__virtualname__ = "vmware_cluster_ha"
__proxyenabled__ = ["vmware_cluster_ha"]


def __virtual__():
    if not HAS_PYVMOMI:
        return False, "Unable to import pyVmomi module."
    return __virtualname__


def _set_slot_based_admission_control_params(cluster_spec, admission_control_policy):
    """
    Set slot based admission control params
    """
    cluster_spec.dasConfig.admissionControlPolicy = (
        vim.cluster.FailoverLevelAdmissionControlPolicy()
    )
    cluster_spec.dasConfig.admissionControlPolicy.failoverLevel = admission_control_policy.get(
        "slot_based_admission_control", {}
    ).get("failover_level")
    cluster_spec.dasConfig.admissionControlPolicy.resourceReductionToToleratePercent = (
        admission_control_policy.get("slot_based_admission_control", {}).get(
            "resource_reduction_to_tolerate_percent"
        )
    )
    cluster_spec.dasConfig.admissionControlEnabled = True


def _set_failover_host_admission_control_params(
    cluster_spec, admission_control_policy, service_instance, datacenter, cluster
):
    """
    Set failover host admission control params
    """
    cluster_spec.dasConfig.admissionControlPolicy = vim.cluster.FailoverHostAdmissionControlPolicy()
    hosts = utils_esxi.get_hosts(
        service_instance=service_instance,
        datacenter_name=datacenter,
        cluster_name=cluster,
        host_names=admission_control_policy.get("failover_host_admission_control", {}).get(
            "failover_hosts"
        ),
    )
    cluster_spec.dasConfig.admissionControlPolicy.failoverHosts = hosts
    cluster_spec.dasConfig.admissionControlPolicy.failoverLevel = admission_control_policy.get(
        "failover_host_admission_control", {}
    ).get("failover_level")
    cluster_spec.dasConfig.admissionControlPolicy.resourceReductionToToleratePercent = (
        admission_control_policy.get("failover_host_admission_control", {}).get(
            "resource_reduction_to_tolerate_percent"
        )
    )
    cluster_spec.dasConfig.admissionControlEnabled = True


def _set_reservation_based_admission_control_params(cluster_spec, admission_control_policy):
    """
    Set reservation based admission control params
    """
    cluster_spec.dasConfig.admissionControlPolicy = (
        vim.cluster.FailoverResourcesAdmissionControlPolicy()
    )
    cluster_spec.dasConfig.admissionControlPolicy.failoverLevel = admission_control_policy.get(
        "reservation_based_admission_control", {}
    ).get("failover_level")
    autocompute_percentages = admission_control_policy.get(
        "reservation_based_admission_control", {}
    ).get("autocompute_percentages")
    cluster_spec.dasConfig.admissionControlPolicy.autoComputePercentages = autocompute_percentages
    if not autocompute_percentages:
        cluster_spec.dasConfig.admissionControlPolicy.cpuFailoverResourcesPercent = (
            admission_control_policy.get("reservation_based_admission_control", {}).get(
                "cpu_failover_resources_percent"
            )
        )
        cluster_spec.dasConfig.admissionControlPolicy.memoryFailoverResourcesPercent = (
            admission_control_policy.get("reservation_based_admission_control", {}).get(
                "memory_failover_resources_percent"
            )
        )
    cluster_spec.dasConfig.admissionControlPolicy.resourceReductionToToleratePercent = (
        admission_control_policy.get("reservation_based_admission_control", {}).get(
            "resource_reduction_to_tolerate_percent"
        )
    )
    cluster_spec.dasConfig.admissionControlEnabled = True


def _set_admission_control_params(
    cluster_spec, admission_control_policy, service_instance, datacenter, cluster
):
    """
    Set admission control params
    """
    cluster_spec.dasConfig.admissionControlEnabled = False
    if "slot_based_admission_control" in admission_control_policy:
        _set_slot_based_admission_control_params(cluster_spec, admission_control_policy)
    elif "failover_host_admission_control" in admission_control_policy:
        _set_failover_host_admission_control_params(
            cluster_spec, admission_control_policy, service_instance, datacenter, cluster
        )

    elif "reservation_based_admission_control" in admission_control_policy:
        _set_reservation_based_admission_control_params(cluster_spec, admission_control_policy)


def configure(
    cluster,
    datacenter,
    enable=False,
    host_monitoring=vim.cluster.DasConfigInfo.ServiceState.enabled,
    vm_monitoring=vim.cluster.DasConfigInfo.VmMonitoringState.vmMonitoringDisabled,
    vm_component_protecting=vim.cluster.DasConfigInfo.ServiceState.disabled,
    vm_min_up_time=120,
    vm_max_failure_window=-1,
    vm_max_failures=3,
    vm_failure_interval=30,
    isolation_response=vim.cluster.DasVmSettings.IsolationResponse.powerOff,
    restart_priority=vim.cluster.DasVmSettings.RestartPriority.medium,
    restart_priority_timeout=120,
    enable_apd_timeout_for_hosts=False,
    vm_reaction_on_apd_cleared="none",
    vm_storage_protection_for_apd="warning",
    vm_storage_protection_for_pdl="warning",
    vm_terminate_delay_for_apd_sec=180,
    admission_control_policy=None,
    advanced_options=None,
    service_instance=None,
    profile=None,
):
    """
    Configure HA for a given cluster

    Supported proxies: esxcluster

    cluster
        The cluster name

    datacenter
        The datacenter name to which the cluster belongs

    enable
        Enable HA for the cluster

    host_monitoring
        Determines whether HA restarts virtual machines after a host fails. Valid values - enabled, disabled. Default - enabled.

    vm_monitoring
        Specifies the level of HA Virtual Machine Health Monitoring Service. Valid values - vmAndAppMonitoring, vmMonitoringDisabled
        and vmMonitoringOnly. Default - vmMonitoringDisabled.

    vm_component_protecting
        Indicates if vSphere HA VM Component Protection service is enabled. Valid values - enabled, disabled. Default - disabled.

    vm_min_up_time
        The number of seconds for the virtual machine's heartbeats to stabilize after the virtual machine has been powered on.
        This time should include the guest operating system boot-up time. The virtual machine monitoring will begin only after this period.
        Default - 120 seconds.

    vm_max_failure_window
        The number of seconds for the window during which up to maxFailures resets can occur before automated responses stop.
        If set to -1, no failure window is specified. Default -1.

    vm_max_failures
        Maximum number of failures and automated resets allowed during the time that maxFailureWindow specifies.
        If maxFailureWindow is -1 (no window), this represents the absolute number of failures after which automated response is stopped.
        If a virtual machine exceeds this threshold, in-depth problem analysis is usually needed.
        The default value is 3.

    vm_failure_interval
        If no heartbeat has been received for at least the specified number of seconds, the virtual machine is declared as failed.
        The default value is 30.

    isolation_response
        Indicates whether or not the virtual machine should be powered off if a host determines that it is isolated from the rest of the compute resource.
        If not specified at either the cluster level or the virtual machine level, this will default to powerOff.

    restart_priority
        Restart priority for a virtual machine.
        If not specified at either the cluster level or the virtual machine level, this will default to medium.

    restart_priority_timeout
        This setting is used to specify a maximum time the lower priority VMs should wait for the higher priority VMs to be ready.
        If the higher priority Vms are not ready by this time, then the lower priority VMs are restarted irrespective of the VM ready state.
        This timeout can be used to prevent the failover of lower priority VMs to be stuck infinitely.
        Default - 120

    enable_apd_timeout_for_hosts
        This property indicates if APD timeout will be enabled for all the hosts in the cluster when vSphere HA is configured.
        Default - False

    vm_reaction_on_apd_cleared
        Action taken by VM Component Protection service for a powered on VM when APD condition clears after APD timeout.
        Default - none

    vm_storage_protection_for_apd
        VM storage protection setting for storage failures categorized as All Paths Down (APD).
        Valid values - disabled, warning, restartConservative, restartAggressive, clusterDefault. Default - warning

    vm_storage_protection_for_pdl
        VM storage protection setting for storage failures categorized as Permenant Device Loss (PDL).
        Valid values - disabled, warning, restartConservative, restartAggressive, clusterDefault. Default - warning

    vm_terminate_delay_for_apd_sec
        The time interval after an APD timeout has been declared and before VM Component Protection service will terminate the VM.
        Default 180 seconds.

    admission_control_policy
        Specify the admission control policy for the cluster as a dictionary.

        .. code-block:: json

            {
              "slot_based_admission_control": {
                "failover_level": 1,
                "resource_reduction_to_tolerate_percent": 20
              }
            }

            {
              "failover_host_admission_control": {
                "failover_level": 10,
                "resource_reduction_to_tolerate_percent": 30,
                "failover_hosts": ["host1", "host2"]
              }
            }

            {
                "reservation_based_admission_control": {
                    "failover_level": 22,
                    "resource_reduction_to_tolerate_percent": 33,
                    "autocompute_percentages": false,
                    "cpu_failover_resources_percent": 45,
                    "memory_failover_resources_percent": 56
                }
            }

    advanced_settings
        Advanced options for the cluster, to be passed in as a dictionary.

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)

    CLI Example:

    .. code-block:: bash

        salt '*' vmware_cluster_ha.configure cluster1 dc1 enable=True
    """
    service_instance = service_instance or connect.get_service_instance(
        config=__opts__, profile=profile
    )
    admission_control_policy = admission_control_policy or {}
    try:
        dc_ref = utils_datacenter.get_datacenter(service_instance, datacenter)
        cluster_ref = utils_cluster.get_cluster(dc_ref=dc_ref, cluster=cluster)
        cluster_spec = vim.cluster.ConfigSpecEx()
        cluster_spec.dasConfig = vim.cluster.DasConfigInfo()
        cluster_spec.dasConfig.enabled = enable
        cluster_spec.dasConfig.hostMonitoring = host_monitoring
        cluster_spec.dasConfig.vmMonitoring = vm_monitoring
        cluster_spec.dasConfig.vmComponentProtecting = vm_component_protecting

        vm_tool_spec = vim.cluster.VmToolsMonitoringSettings()
        vm_tool_spec.vmMonitoring = vm_monitoring
        vm_tool_spec.minUpTime = vm_min_up_time
        vm_tool_spec.maxFailureWindow = vm_max_failure_window
        vm_tool_spec.maxFailures = vm_max_failures
        vm_tool_spec.failureInterval = vm_failure_interval

        das_spec = vim.cluster.DasVmSettings()
        das_spec.isolationResponse = isolation_response
        das_spec.restartPriority = restart_priority
        das_spec.restartPriorityTimeout = restart_priority_timeout
        das_spec.vmToolsMonitoringSettings = vm_tool_spec

        component_protection_spec = vim.cluster.VmComponentProtectionSettings()
        component_protection_spec.enableAPDTimeoutForHosts = enable_apd_timeout_for_hosts
        component_protection_spec.vmReactionOnAPDCleared = vm_reaction_on_apd_cleared
        component_protection_spec.vmStorageProtectionForAPD = vm_storage_protection_for_apd
        component_protection_spec.vmStorageProtectionForPDL = vm_storage_protection_for_pdl
        component_protection_spec.vmTerminateDelayForAPDSec = vm_terminate_delay_for_apd_sec
        das_spec.vmComponentProtectionSettings = component_protection_spec

        _set_admission_control_params(
            cluster_spec=cluster_spec,
            admission_control_policy=admission_control_policy,
            service_instance=service_instance,
            datacenter=datacenter,
            cluster=cluster,
        )

        cluster_spec.dasConfig.defaultVmSettings = das_spec

        cluster_spec.dasConfig.option = []
        for key in advanced_options or {}:
            cluster_spec.dasConfig.option.append(
                vim.OptionValue(key=key, value=advanced_options[key])
            )
        utils_cluster.update_cluster(cluster_ref=cluster_ref, cluster_spec=cluster_spec)
    except (salt.exceptions.VMwareApiError, salt.exceptions.VMwareRuntimeError) as exc:
        return {cluster: False, "reason": str(exc)}
    return {cluster: True}


def get(cluster_name, datacenter_name, service_instance=None, profile=None):
    """
    Get HA info about a cluster in a datacenter

    cluster_name
        The cluster name

    datacenter_name
        The datacenter name to which the cluster belongs

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)

    CLI Example:

    .. code-block:: bash

        salt '*' vmware_cluster_ha.get cluster_name=cl1 datacenter_name=dc1
    """
    ret = {}
    service_instance = service_instance or connect.get_service_instance(
        config=__opts__, profile=profile
    )
    try:
        dc_ref = utils_datacenter.get_datacenter(service_instance, datacenter_name)
        cluster_ref = utils_cluster.get_cluster(dc_ref=dc_ref, cluster=cluster_name)
        das_config = cluster_ref.configurationEx.dasConfig
        ret["enabled"] = das_config.enabled
        ret["host_monitoring"] = das_config.hostMonitoring
        ret["vm_monitoring"] = das_config.vmMonitoring
        ret["vm_component_protecting"] = das_config.vmComponentProtecting

        if das_config.defaultVmSettings:
            vm_tools_monitoring_settings = das_config.defaultVmSettings.vmToolsMonitoringSettings
            ret["vm_monitoring"] = vm_tools_monitoring_settings.vmMonitoring
            ret["vm_min_up_time"] = vm_tools_monitoring_settings.minUpTime
            ret["vm_max_failure_window"] = vm_tools_monitoring_settings.maxFailureWindow
            ret["vm_max_failures"] = vm_tools_monitoring_settings.maxFailures
            ret["vm_failure_interval"] = vm_tools_monitoring_settings.failureInterval
            ret["isolation_response"] = das_config.defaultVmSettings.isolationResponse
            ret["restart_priority"] = das_config.defaultVmSettings.restartPriority
            ret["restart_priority_timeout"] = das_config.defaultVmSettings.restartPriorityTimeout

            component_protection_settings = (
                das_config.defaultVmSettings.vmComponentProtectionSettings
            )
            ret[
                "enable_apd_timeout_for_hosts"
            ] = component_protection_settings.enableAPDTimeoutForHosts
            ret["vm_reaction_on_apd_cleared"] = component_protection_settings.vmReactionOnAPDCleared
            ret[
                "vm_storage_protection_for_apd"
            ] = component_protection_settings.vmStorageProtectionForAPD
            ret[
                "vm_storage_protection_for_pdl"
            ] = component_protection_settings.vmStorageProtectionForPDL
            ret[
                "vm_terminate_delay_for_apd_sec"
            ] = component_protection_settings.vmTerminateDelayForAPDSec

            ret["admission_control_enabled"] = das_config.admissionControlEnabled
            ret["admission_control_policy"] = None
            ret["failover_level"] = das_config.admissionControlPolicy.failoverLevel
            ret[
                "resource_reduction_to_tolerate_percent"
            ] = das_config.admissionControlPolicy.resourceReductionToToleratePercent
            if isinstance(
                das_config.admissionControlPolicy,
                vim.cluster.FailoverLevelAdmissionControlPolicy,
            ):
                ret["admission_control_policy"] = "slot_based_admission_control"
            elif isinstance(
                das_config.admissionControlPolicy,
                vim.cluster.FailoverHostAdmissionControlPolicy,
            ):
                ret["admission_control_policy"] = "failover_host_admission_control"
                ret["failover_hosts"] = [
                    h.name for h in das_config.admissionControlPolicy.failoverHosts
                ]
            elif isinstance(
                das_config.admissionControlPolicy,
                vim.cluster.FailoverResourcesAdmissionControlPolicy,
            ):
                ret["admission_control_policy"] = "reservation_based_admission_control"
                ret[
                    "autocompute_percentages"
                ] = das_config.admissionControlPolicy.autoComputePercentages
                ret[
                    "cpu_failover_resources_percent"
                ] = das_config.admissionControlPolicy.cpuFailoverResourcesPercent
                ret[
                    "memory_failover_resources_percent"
                ] = das_config.admissionControlPolicy.memoryFailoverResourcesPercent

        ret["advanced_settings"] = {}
        for obj in cluster_ref.configurationEx.dasConfig.option:
            ret["advanced_settings"][obj.key] = obj.value
    except (salt.exceptions.VMwareApiError, salt.exceptions.VMwareRuntimeError) as exc:
        return {cluster_name: False, "reason": str(exc)}
    return ret



# pylint: disable=missing-module-docstring
from pathlib import Path

import setuptools

version_path = Path(__file__).parent / "src" / "saltext" / "vmware" / "version.py"

with version_path.open() as f:
    for line in f:
        if line.startswith("__version__"):
            # We only want the bare string pls
            version = line.partition("=")[-1].strip().strip('"').strip("'")
            break
    else:
        version = "0.0.1dev1"


if __name__ == "__main__":
    setuptools.setup(use_scm_version=True, version=version)



"""
Salt execution module for VMC vm stats
Provides methods to get cpu or memory related settings of a virtual machine
"""
import logging

from saltext.vmware.utils import vmc_constants
from saltext.vmware.utils import vmc_request
from saltext.vmware.utils import vmc_vcenter_request

log = logging.getLogger(__name__)

__virtualname__ = "vmc_vm_stats"


def __virtual__():
    return __virtualname__


def get(hostname, username, password, vm_id, stats_type, verify_ssl=True, cert=None):
    """
    Retrieves the cpu or memory related settings for given VM.

    CLI Example:

    .. code-block:: bash

        salt vm_minion vmc_vm_stats.get hostname=sample-vcenter.vmwarevmc.com ...

    hostname
        Hostname of the vCenter console

    username
        username required to login to vCenter console

    password
        password required to login to vCenter console

    vm_id
        Virtual machine identifier for which the stats should be retrieved.

    stats_type
        Type of the stats to be retrieved for given VM. Possible values: cpu, memory

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to VMC Cloud Console.
        The certificate can be retrieved from browser.

    """

    log.info("Retrieving the %s related settings for VM %s", stats_type, vm_id)
    api_url_base = vmc_request.set_base_url(hostname)
    api_url = "{base_url}api/vcenter/vm/{vm_id}/hardware/{stats_type}"
    api_url = api_url.format(base_url=api_url_base, vm_id=vm_id, stats_type=stats_type)

    headers = vmc_vcenter_request.get_headers(hostname, username, password)
    return vmc_vcenter_request.call_api(
        method=vmc_constants.GET_REQUEST_METHOD,
        url=api_url,
        headers=headers,
        description="vmc_vm_stats.get",
        verify_ssl=verify_ssl,
        cert=cert,
    )



"""
Salt State file to create/update/delete an IP Address Pool

Example:

.. code-block:: yaml

    create_ip_pool:
      nsxt_ip_pools.present:
        - name: Create IP Pool
          hostname: {{ pillar['nsxt_manager_hostname'] }}
          username: {{ pillar['nsxt_manager_username'] }}
          password: {{ pillar['nsxt_manager_password'] }}
          cert: {{ pillar['nsxt_manager_certificate'] }}
          verify_ssl: <False/True>
          display_name: <ip pool name>
          description: <ip pool description>
          tags:
            - tag: <tag-key-1>
              scope: <tag-value-1>
            - tag: <tag-key-2>
              scope: <tag-value-2>
          subnets:
            - cidr: <cidr_value>
              gateway_ip: <gateway_ip_value>
              dns_nameservers:
                - <dns_nameserver1>
                - <dns_nameserver2>
              allocation_ranges:
                - "start": <IP-Address-Range-start-1>
                  "end": <IP-Address-Range-end-1>
          ip_release_delay: <delay in milliseconds>
"""
import logging

log = logging.getLogger(__name__)

__virtual_name__ = "nsxt_ip_pools"


def __virtual__():
    """
    Only load if the nsxt_ip_pools module is available in __salt__
    """
    return (
        __virtual_name__ if "nsxt_ip_pools.get" in __salt__ else False,
        "'nsxt_ip_pools' binary not found on system",
    )


def _create_state_response(name, old_state, new_state, result, comment):
    state_response = dict()
    state_response["name"] = name
    state_response["result"] = result
    state_response["comment"] = comment
    state_response["changes"] = dict()
    if new_state or old_state:
        state_response["changes"]["old"] = old_state
        state_response["changes"]["new"] = new_state

    return state_response


def _needs_update(existing_ip_pool, input_dict):
    updatable_keys = ["subnets", "description", "tags", "ip_release_delay"]

    is_updatable = False

    # check if any updatable field has different value from the existing one
    for key in updatable_keys:
        existing_val = existing_ip_pool.get(key)
        input_val = input_dict.get(key)
        if not existing_val and input_val:
            is_updatable = True
        if existing_val and input_val and existing_val != input_val:
            is_updatable = True

    return is_updatable


def _fill_input_dict_with_existing_info(existing_ip_pool, input_dict):
    for key in dict(existing_ip_pool).keys():
        if key not in input_dict:
            input_dict[key] = existing_ip_pool[key]


def present(
    name,
    display_name,
    hostname,
    username,
    password,
    cert=None,
    verify_ssl=True,
    cert_common_name=None,
    description=None,
    tags=None,
    subnets=None,
    ip_release_delay=None,
):
    """
    Creates/Updates(if present with the same name) an IP Address Pool

    .. code-block:: yaml

        create_ip_pool:
          nsxt_ip_pools.present:
            - name: Create IP Pool
              hostname: {{ pillar['nsxt_manager_hostname'] }}
              username: {{ pillar['nsxt_manager_username'] }}
              password: {{ pillar['nsxt_manager_password'] }}
              cert: {{ pillar['nsxt_manager_certificate'] }}
              verify_ssl: <False/True>
              display_name: <ip pool name>
              description: <ip pool description>
              tags:
                - tag: <tag-key-1>
                  scope: <tag-value-1>
                - tag: <tag-key-2>
                  scope: <tag-value-2>
              subnets:
                - cidr: <cidr_value>
                  gateway_ip: <gateway_ip_value>
                  dns_nameservers:
                    - <dns_nameserver1>
                    - <dns_nameserver2>
                  allocation_ranges:
                    - start: <IP-Address-Range-start-1>
                      end: <IP-Address-Range-end-1>
              ip_release_delay: <delay in milliseconds>

    name
        The Operation to perform

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against

    display_name
        The name using which IP Address Pool will be created. In case of update the provided display_name should be
        unique else the method will raise an error

    description
        (Optional) description for the IP Address Pool

    tags
        (Optional) Opaque identifiers meaningful to the API user. Maximum 30 tags can be associated:

        .. code-block:: yaml

            tags:
                - tag: <tag-key-1>
                  scope: <tag-value-1>
                - tag: <tag-key-2>
                  scope: <tag-value-2>

    subnets
        (Optional) The collection of one or more subnet objects in a pool.
        Subnets can be IPv4 or IPv6 and they should not overlap. The maximum number will not exceed 5 subnets.

        .. code-block:: yaml

            subnets:
                - cidr: <cidr_value>
                  gateway_ip: <gateway_ip_value>
                  dns_nameservers:
                    - <dns_nameserver1>
                      <dns_nameserver2>
                  allocation_ranges:
                    - start: <IP-Address-Range-start-1>
                      end: <IP-Address-Range-end-1>

    ip_release_delay
        (Optional) Delay in milliseconds, while releasing allocated IP address from IP pool (Default is 2 mins - configured on NSX device).
    """
    input_dict = {
        "display_name": display_name,
        "description": description,
        "tags": tags,
        "subnets": subnets,
        "ip_release_delay": ip_release_delay,
    }

    log.info("Checking if IP Pool with name %s is present", display_name)
    get_ip_pools_response = __salt__["nsxt_ip_pools.get_by_display_name"](
        hostname,
        username,
        password,
        display_name,
        cert=cert,
        verify_ssl=verify_ssl,
        cert_common_name=cert_common_name,
    )

    if "error" in get_ip_pools_response:
        return _create_state_response(name, None, None, False, get_ip_pools_response["error"])

    ip_pools = get_ip_pools_response["results"]

    if len(ip_pools) > 1:
        log.info("Multiple instances found for the provided display name %s", display_name)
        return _create_state_response(
            name,
            None,
            None,
            False,
            "Multiple IP Pools found for the provided display name {}".format(display_name),
        )

    existing_ip_pool = ip_pools[0] if len(ip_pools) > 0 else None

    if __opts__.get("test"):
        log.info("present is called with test option")
        if existing_ip_pool:
            return _create_state_response(
                name,
                None,
                None,
                None,
                "State present will update IP Pool with name {}".format(display_name),
            )
        else:
            return _create_state_response(
                name,
                None,
                None,
                None,
                "State present will create IP Pool with name {}".format(display_name),
            )
    if existing_ip_pool:
        is_update_required = _needs_update(existing_ip_pool, input_dict)

        if is_update_required:
            _fill_input_dict_with_existing_info(existing_ip_pool, input_dict)

            log.info("IP Pool found with name %s", display_name)
            updated_ip_pool = __salt__["nsxt_ip_pools.update"](
                ip_pool_id=existing_ip_pool["id"],
                revision=existing_ip_pool["_revision"],
                hostname=hostname,
                username=username,
                password=password,
                cert=cert,
                verify_ssl=verify_ssl,
                cert_common_name=cert_common_name,
                display_name=input_dict.get("display_name"),
                description=input_dict.get("description"),
                tags=input_dict.get("tags"),
                subnets=input_dict.get("subnets"),
                ip_release_delay=input_dict.get("ip_release_delay"),
            )

            if "error" in updated_ip_pool:
                return _create_state_response(name, None, None, False, updated_ip_pool["error"])

            return _create_state_response(
                name,
                existing_ip_pool,
                updated_ip_pool,
                True,
                "Updated IP Pool {}".format(display_name),
            )
        else:
            log.info("All fields are same as existing IP Address Pool %s", display_name)
            return _create_state_response(
                name, None, None, True, "IP Address Pool exists already, no action to perform"
            )
    else:
        log.info("No IP Pool found with name %s", display_name)
        created_ip_pool = __salt__["nsxt_ip_pools.create"](
            hostname,
            username,
            password,
            display_name=input_dict.get("display_name"),
            cert=cert,
            verify_ssl=verify_ssl,
            cert_common_name=cert_common_name,
            description=input_dict.get("description"),
            tags=input_dict.get("tags"),
            subnets=input_dict.get("subnets"),
            ip_release_delay=input_dict.get("ip_release_delay"),
        )

        if "error" in created_ip_pool:
            return _create_state_response(name, None, None, False, created_ip_pool["error"])

        return _create_state_response(
            name, None, created_ip_pool, True, "Created IP Pool {}".format(display_name)
        )


def absent(
    name,
    display_name,
    hostname,
    username,
    password,
    cert=None,
    verify_ssl=True,
    cert_common_name=None,
):
    """
    Deletes an IP Address Pool of provided name (if present)

    .. code-block:: yaml

    delete_ip_pool:
      nsxt_ip_pools.absent:
        - name: Delete IP Pool
          hostname: {{ pillar['nsxt_manager_hostname'] }}
          username: {{ pillar['nsxt_manager_username'] }}
          password: {{ pillar['nsxt_manager_password'] }}
          cert: {{ pillar['nsxt_manager_certificate'] }}
          verify_ssl: <False/True>
          display_name: <ip pool name>

    name
        The Operation to perform

    hostname
        The host name of NSX-T manager

    username
        Username to connect to NSX-T manager

    password
        Password to connect to NSX-T manager

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to NSX-T manager.
        The certificate can be retrieved from browser.

    cert_common_name
        (Optional) By default, the hostname parameter and the common name in certificate is compared for host name
        verification. If the client certificate common name and hostname do not match (in case of self-signed
        certificates), specify the certificate common name as part of this parameter. This value is then used to
        compare against

    display_name
        Display name of IP Address Pool to delete
    """

    log.info("Checking if IP Address Pool with name %s is present", display_name)

    get_ip_pools_response = __salt__["nsxt_ip_pools.get_by_display_name"](
        hostname,
        username,
        password,
        display_name,
        cert=cert,
        verify_ssl=verify_ssl,
        cert_common_name=cert_common_name,
    )

    if "error" in get_ip_pools_response:
        return _create_state_response(name, None, None, False, get_ip_pools_response["error"])

    ip_pools = get_ip_pools_response["results"]

    if len(ip_pools) > 1:
        log.info("Multiple instances found for the provided display name %s", display_name)
        return _create_state_response(
            name,
            None,
            None,
            False,
            "Multiple IP Pools found for the provided display name {}".format(display_name),
        )

    existing_ip_pool = ip_pools[0] if len(ip_pools) > 0 else None

    if __opts__.get("test"):
        log.info("absent is called with test option")
        if existing_ip_pool:
            return _create_state_response(
                name,
                None,
                None,
                None,
                "State absent will delete IP Pool with name {}".format(display_name),
            )
        else:
            return _create_state_response(
                name,
                None,
                None,
                None,
                "State absent will do nothing as no IP Pool found with name {}".format(
                    display_name
                ),
            )

    if existing_ip_pool:
        log.info("IP Address Pool found with name %s", display_name)
        deleted_response = __salt__["nsxt_ip_pools.delete"](
            existing_ip_pool["id"],
            hostname,
            username,
            password,
            cert=cert,
            verify_ssl=verify_ssl,
            cert_common_name=cert_common_name,
        )

        if "error" in deleted_response:
            return _create_state_response(name, None, None, False, deleted_response["error"])

        return _create_state_response(
            name, existing_ip_pool, None, True, "Deleted IP Pool {}".format(display_name)
        )
    else:
        log.info("No IP Address Pool found with name %s", display_name)
        return _create_state_response(
            name, None, None, True, "No IP Address Pool found with name {}".format(display_name)
        )



# Copyright 2022 VMware, Inc.
# SPDX-License-Identifier: Apache-2.0
import json
import logging

import saltext.vmware.utils.connect as connect
import saltext.vmware.utils.vsphere as utils_vmware

log = logging.getLogger(__name__)

try:
    from pyVmomi import vim, vmodl, VmomiSupport

    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False

__virtualname__ = "vmware_dvportgroup"
__proxyenabled__ = ["vmware_dvportgroup"]


def __virtual__():
    if not HAS_PYVMOMI:
        return False, "Unable to import pyVmomi module."
    return __virtualname__


def get(switch_name, portgroup_key, host_name=None, service_instance=None, profile=None):
    """
    Get distributed portgroup from a distributed switch optionally from a host.

    service_instance
        Service instance object to access vCenter

    switch_name
        Name of the distributed switch.

    portgroup_key
        Portgroup key.

    host_name
        Name of the ESXi host.  (optional).

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional)

    profile
        Profile to use (optional)
    """
    ret = {}

    service_instance = service_instance or connect.get_service_instance(
        config=__opts__, profile=profile
    )

    switch_ref = utils_vmware._get_dvs(service_instance=service_instance, dvs_name=switch_name)

    if switch_ref:
        for portgroup in switch_ref.portgroup:
            if portgroup.key == portgroup_key:
                ret["name"] = portgroup.config.name
                ret["vlan"] = portgroup.config.defaultPortConfig.vlan.vlanId
                ret["pnic"] = []
                if host_name:
                    for host in portgroup.config.distributedVirtualSwitch.config.host:
                        if host.config.host.name == host_name:
                            for pnic in host.config.backing.pnicSpec:
                                ret["pnic"].append(pnic.pnicDevice)

        ret = json.loads(json.dumps(ret, cls=VmomiSupport.VmomiJSONEncoder))
    return ret



# Copyright 2021 VMware, Inc.
# SPDX-License: Apache-2.0
import logging
import os

import salt.exceptions
import saltext.vmware.utils.common as utils_common
import saltext.vmware.utils.connect as utils_connect
import saltext.vmware.utils.esxi as utils_esxi
import saltext.vmware.utils.vsphere as utils_vsphere


log = logging.getLogger(__name__)

try:
    from pyVmomi import vmodl, vim, VmomiSupport

    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False


__virtualname__ = "vmware_vsphere"

DEFAULT_EXCEPTIONS = (
    vim.fault.InvalidState,
    vim.fault.NotFound,
    vim.fault.HostConfigFault,
    vmodl.fault.InvalidArgument,
    salt.exceptions.VMwareApiError,
    vim.fault.AlreadyExists,
    vim.fault.UserNotFound,
    salt.exceptions.CommandExecutionError,
    vmodl.fault.SystemError,
    TypeError,
)


def __virtual__():
    if not HAS_PYVMOMI:
        return False, "Unable to import pyVmomi module."
    return __virtualname__


def system_info(
    service_instance=None,
    profile=None,
):
    """
    .. versionadded:: 23.4.4.0rc1

    Return system information about a VMware environment.

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)

    .. code-block:: bash

        salt '*' vmware_vsphere.system_info
    """
    log.debug("Running vmware_vsphere.system_info")
    ret = None
    service_instance = service_instance or utils_connect.get_service_instance(
        config=__opts__, profile=profile
    )
    ret = utils_common.get_inventory(service_instance).about.__dict__
    if "apiType" in ret:
        if ret["apiType"] == "HostAgent":
            ret = dictupdate.update(ret, utils_common.get_hardware_grains(service_instance))
    return ret


def list_resourcepools(
    service_instance=None,
    profile=None,
):
    """
    .. versionadded:: 23.4.4.0rc1

    Returns a list of resource pools for the specified host.

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)

    CLI Example:

    .. code-block:: bash

        salt '*' vmware_vsphere.list_resourcepools
    """
    log.debug("Running vmware_vsphere.list_resourcepools")
    service_instance = service_instance or utils_connect.get_service_instance(
        config=__opts__, profile=profile
    )
    return utils_vsphere.list_resourcepools(service_instance)


def list_networks(
    service_instance=None,
    profile=None,
):
    """
    .. versionadded:: 23.4.4.0rc1

    Returns a list of networks for the specified host.

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)

    CLI Example:

    .. code-block:: bash

        salt '*' vmware_vsphere.list_networks
    """
    log.debug("Running vmware_vsphere.list_networks")
    service_instance = service_instance or utils_connect.get_service_instance(
        config=__opts__, profile=profile
    )
    return utils_vsphere.list_networks(service_instance)


def list_vapps(
    service_instance=None,
    profile=None,
):
    """
    .. versionadded:: 23.6.29.0rc1

    Returns a list of vApps for the specified host.

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)

    CLI Example:

    .. code-block:: bash

        # List vapps from all minions
        salt '*' vmware_vsphere.list_vapps
    """
    log.debug("Running vmware_vsphere.list_vapps")
    service_instance = service_instance or utils_connect.get_service_instance(
        config=__opts__, profile=profile
    )
    return utils_vsphere.list_vapps(service_instance)


def _get_host_disks(host_reference):
    """
    Helper function that returns a dictionary containing a list of SSD and Non-SSD disks.
    """
    storage_system = host_reference.configManager.storageSystem
    disks = storage_system.storageDeviceInfo.scsiLun
    ssds = []
    non_ssds = []

    for disk in disks:
        try:
            has_ssd_attr = disk.ssd
        except AttributeError:
            has_ssd_attr = False
        if has_ssd_attr:
            ssds.append(disk)
        else:
            non_ssds.append(disk)

    return {"SSDs": ssds, "Non-SSDs": non_ssds}


def list_ssds(
    host_name=None,
    datacenter_name=None,
    cluster_name=None,
    service_instance=None,
    profile=None,
):
    """
    .. versionadded:: 23.6.29.0rc1

    Returns a list of SSDs for the given host or list of host_names.

    datacenter_name
        Filter by this datacenter name (required when cluster is specified)

    cluster_name
        Filter by this cluster name (optional)

    host_name
        Filter by this ESXi hostname (optional)

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)

    CLI Example:

    .. code-block:: bash

        salt '*' vmware_vsphere.list_ssds
    """
    log.debug("Running vmware_vsphere.list_ssds")
    ret = {}
    service_instance = service_instance or utils_connect.get_service_instance(
        config=__opts__, profile=profile
    )
    hosts = utils_esxi.get_hosts(
        service_instance=service_instance,
        host_names=[host_name] if host_name else None,
        cluster_name=cluster_name,
        datacenter_name=datacenter_name,
        get_all_hosts=host_name is None,
    )
    for host in hosts:
        disks = _get_host_disks(host).get("SSDs")
        names = []
        for disk in disks:
            names.append(disk.canonicalName)
        ret.update({host.name: names})

    return ret


def list_non_ssds(
    host_name=None,
    datacenter_name=None,
    cluster_name=None,
    service_instance=None,
    profile=None,
):
    """
    .. versionadded:: 23.6.29.0rc1

    Returns a list of Non-SSD disks for the given host or list of host_names.

    .. note::

        In the pyVmomi StorageSystem, ScsiDisks may, or may not have an ``ssd`` attribute.
        This attribute indicates if the ScsiDisk is SSD backed. As this option is optional,
        if a relevant disk in the StorageSystem does not have ``ssd = true``, it will end
        up in the ``non_ssds`` list here.

    host_name
        Filter by this ESXi hostname (optional)

    datacenter_name
        Filter by this datacenter name (required when cluster is specified)

    cluster_name
        Filter by this cluster name (optional)

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)

    CLI Example:

    .. code-block:: bash

        salt '*' vmware_vsphere.list_non_ssds
    """
    log.debug("Running vmware_vsphere.list_non_ssds")
    ret = {}
    service_instance = service_instance or utils_connect.get_service_instance(
        config=__opts__, profile=profile
    )
    hosts = utils_esxi.get_hosts(
        service_instance=service_instance,
        host_names=[host_name] if host_name else None,
        cluster_name=cluster_name,
        datacenter_name=datacenter_name,
        get_all_hosts=host_name is None,
    )
    for host in hosts:
        names = []
        disks = _get_host_disks(host).get("Non-SSDs")
        for disk in disks:
            names.append(disk.canonicalName)
        ret.update({host.name: names})

    return ret



# SPDX-License-Identifier: Apache-2.0
import logging

import saltext.vmware.utils.connect as connect
import saltext.vmware.utils.datastore as utils_datastore

log = logging.getLogger(__name__)

try:
    from pyVmomi import vim

    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False


__virtualname__ = "vmware_datastore"
__proxyenabled__ = ["vmware_datastore"]


def __virtual__():
    if not HAS_PYVMOMI:
        return False, "Unable to import pyVmomi module."
    return __virtualname__


def maintenance_mode(
    name, enter_maintenance_mode, datacenter_name=None, service_instance=None, profile=None
):
    """
    Manage boot option for a virtual machine

    name
        The name of the datastore.

    enter_maintenance_mode
        (Bool) True to put datastore in maintenance mode, False to exit maintenance mode.

    datacenter_name
        The name of the datacenter containing the datastore.

    service_instance
        Use this vCenter service connection instance instead of creating a new one. (optional).

    profile
        Profile to use (optional)
    """
    service_instance = service_instance or connect.get_service_instance(
        config=__opts__, profile=profile
    )
    ret = {"name": name, "changes": {}, "result": True, "comment": ""}
    assert isinstance(name, str)
    datastores = utils_datastore.get_datastores(
        service_instance, datastore_name=name, datacenter_name=datacenter_name
    )
    ds = datastores[0] if datastores else None
    status = ds.summary.maintenanceMode
    if enter_maintenance_mode:
        if status == "inMaintenance":
            ret["comment"] = "Already in maintenance mode."
            return ret

        if __opts__["test"]:
            ret["result"] = None
            ret["changes"] = {"new": f"datastore {name} will enter maintenance mode."}
            ret["comment"] = "These options are set to change."
            return ret

        mode = utils_datastore.enter_maintenance_mode(ds)
        if mode:
            ret["changes"] = {"new": f"datastore {name} is in maintenance mode."}
            ret["comment"] = "These options changed."
            return ret
        ret["comment"] = f"Failed to put datastore {name} in maintenance mode."
        ret["result"] = False
        return ret
    else:
        if status == "normal":
            ret["comment"] = "Already exited maintenance mode."
            return ret

        if __opts__["test"]:
            ret["result"] = None
            ret["changes"] = {"new": f"datastore {name} will exit maintenance mode."}
            ret["comment"] = "These options are set to change."
            return ret

        mode = utils_datastore.exit_maintenance_mode(ds)
        if mode:
            ret["changes"] = {"new": f"datastore {name} exited maintenance mode."}
            ret["comment"] = "These options changed."
            return ret
        ret["comment"] = f"datastore {name} failed to exit maintenance mode."
        ret["result"] = False
        return ret



"""
    NSX-T API Request Module
"""
import json
import logging

import requests
from requests.auth import HTTPBasicAuth
from requests.exceptions import HTTPError
from requests.exceptions import RequestException
from requests.exceptions import SSLError
from saltext.vmware.modules.ssl_adapter import HostHeaderSSLAdapter

log = logging.getLogger(__name__)


def call_api(
    method,
    url,
    username,
    password,
    cert_common_name=None,
    verify_ssl=True,
    cert=None,
    data=None,
    params=None,
):
    headers = {"Accept": "application/json", "content-Type": "application/json"}
    session = requests.Session()

    if cert_common_name and verify_ssl:
        session.mount("https://", HostHeaderSSLAdapter())
        headers["Host"] = cert_common_name

    verify = verify_ssl
    if verify_ssl:
        if cert:
            verify = cert
        else:
            result = {
                "error": "No certificate path specified. Please specify certificate path in cert parameter"
            }
            return result

    try:
        response = session.request(
            method=method,
            url=url,
            headers=headers,
            auth=HTTPBasicAuth(username, password),
            verify=verify,
            params=params,
            data=json.dumps(data),
        )
        log.info("Response status code: {}".format(response.status_code))
        # raise error for any client/server HTTP Error codes
        response.raise_for_status()

        if not response.text:
            return None
        return response.json()

    except HTTPError as e:
        log.error(e)
        result = {
            "error": "Error occurred while calling NSX-T API {}. Please check logs for more details.".format(
                url
            )
        }
        # if response contains json, extract error message from it
        if e.response.text:
            log.error("Response from NSX-T Manager %s", e.response.text)
            try:
                error_json = e.response.json()
                if error_json["error_message"]:
                    result["error"] = e.response.json()["error_message"]
            except ValueError:
                log.error(
                    "Couldn't parse the response as json. Returning response text as error message"
                )
                result["error"] = e.response.text
        return result
    except SSLError as se:
        log.error(se)
        result = {
            "error": "SSL Error occurred while calling NSX-T API {}."
            "Please check if the certificate is valid and hostname matches certificate common name.".format(
                url
            )
        }
        return result
    except RequestException as re:
        log.error(re)
        result = {
            "error": "Error occurred while calling NSX-T API {}. Please check logs for more details.".format(
                url
            )
        }
        return result



"""
VMC-SDDC state module

Add new SDDC and delete existing SDDC.

Example usage :

.. code-block:: yaml

    ensure_sddc_exists:
      vmc_sddc.present:
        - hostname: stg.skyscraper.vmware.com
        - refresh_key: 7jPSGSZpCa8e5Ouks4UY5cZyOtynAhF
        - authorization_host: console-stg.cloud.vmware.com
        - org_id: 10e1092f-51d0-473a-80f8-137652fd0c39
        - sddc_id: 15251c47-bad0-4adf-b8f2-85eba94a2a2f
        - sddc_name: test-1234
        - verify_ssl: False
        - cert: /path/to/client/certificate

    sddc_test-id:
      vmc_sddc.absent:
        - hostname: stg.skyscraper.vmware.com
        - refresh_key: 7jPSGSZpCa8e5Ouks4UY5cZyOtynAhF
        - authorization_host: console-stg.cloud.vmware.com
        - org_id: 10e1092f-51d0-473a-80f8-137652fd0c39

.. warning::

    It is recommended to pass the VMC authentication details using Pillars rather than specifying as plain text in SLS
    files.

"""
import logging

from saltext.vmware.utils import vmc_state

log = logging.getLogger(__name__)


def present(
    name,
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    num_hosts,
    provider,
    region,
    account_link_config=None,
    account_link_sddc_config=None,
    deployment_type=None,
    host_instance_type=None,
    msft_license_config=None,
    sddc_id=None,
    sddc_template_id=None,
    sddc_type=None,
    size=None,
    skip_creating_vxlan=False,
    sso_domain=None,
    storage_capacity=None,
    vpc_cidr=None,
    vxlan_subnet=None,
    validate_only=False,
    verify_ssl=True,
    cert=None,
):
    """
    Ensure a given SDDC exists for given organization.

    name
        The name of SDDC that will be assigned to new created SDDC.

    hostname
        The host name of VMC.

    refresh_key
        API Token of the user which is used to get the Access Token required for VMC operations

    authorization_host
        Hostname of the VMC cloud console.

    org_id
        The ID of organization to which the SDDC belongs to.

    num_hosts
        An integer value to indicate number of hosts in a SDDC.

    provider: String
        Determines what additional properties are available based on cloud provider.
        Possible values are: AWS, ZEROCLOUD

    region: String
        AWS region on which SDDC will be deployed.

    account_link_config
        (Optional) The account linking configuration, we will keep this one and remove accountLinkSddcConfig finally.

        account_link_config expects to be passed as a dict having delay_account_link parameter

        delay_account_link: Boolean
        (Optional) Boolean flag identifying whether account linking should be delayed or not for the SDDC.

        .. code:: yaml

            account_link_config:
              delay_account_link: False

    account_link_sddc_config
        (Optional) A list of the AccountLinkSddcConfig which indicates SDDC linking configurations to use.

        AccountLinkSddcConfig has two parameters connected_account_id  and customer_subnet_ids

        'connected_account_id': String
            (Optional) The ID of the customer connected account to work with.

        'customer_subnet_ids': Array of String
            (Optional) List of subnet Ids

    deployment_type: String
        (Optional) Denotes if request is for a SingleAZ or a MultiAZ SDDC. Default is SingleAZ.
        Possible values are: SingleAZ, MultiAZ

    host_instance_type: String
        (Optional) The instance type for the hosts in the primary cluster of the SDDC.
        Possible values are: i3.metal, r5.metal, i3en.metal

    msft_license_config :
        (Optional) Indicates the desired licensing support, if any, of Microsoft software.
        It can be specified in the below format:

        .. code:: yaml

            msft_license_config:
              academic_license: False
              mssql_licensing: "DISABLED"
              windows_licensing: "DISABLED"

        Possible values are:

        **academic_license**

        ``True`` if it is Academic Standard or ``False`` Commercial Standard License.

        **mssql_licensing**

        The status MSSQL licensing for this SDDC’s clusters. Possible values are: DISABLED, CUSTOMER_SUPPLIED, ENABLED

        **windows_licensing**

        The status of Windows licensing for this SDDC’s clusters. Possible values are: DISABLED, CUSTOMER_SUPPLIED, ENABLED
        Possible values are: DISABLED, CUSTOMER_SUPPLIED, ENABLED

        Please refer the `VMC Doc about msft_license_config <https://developer.vmware.com/apis/vmc/v1.1/data-structures/MsftLicensingConfig/>`_

    sddc_id: String As UUID
        (Optional) If provided, will be assigned as SDDC id of the provisioned SDDC.

    sddc_template_id : String As UUID
        (Optional) If provided, configuration from the template will applied to the provisioned SDDC.

    sddc_type: String
        (Optional) Denotes the SDDC type, if the value is null or empty, the type is considered as default.

    size: String
        (Optional) The size of the vCenter and NSX appliances. “large” sddcSize corresponds to a ‘large’ vCenter
        appliance and ‘large’ NSX appliance. ‘medium’ sddcSize corresponds to ‘medium’ vCenter appliance and
        ‘medium’ NSX appliance. Value defaults to ‘medium’.
        Possible values are: nsx_small, medium, large, nsx_large

    skip_creating_vxlan : Boolean
        (Optional) skip creating vxlan for compute gateway for SDDC provisioning

    sso_domain : String
        (Optional) The SSO domain name to use for vSphere users. If not specified, vmc.local will be used.

    storage_capacity:  Integer As Int64
        (Optional) The storage capacity value to be requested for the SDDC primary cluster, in GiBs. If provided,
        instead of using the direct-attached storage, a capacity value amount of separable storage will be used.

    vpc_cidr
        (Optional) AWS VPC IP range. Only prefix of 16 or 20 is currently supported. Example: 10.2.0.0/16, 10.2.32.0/20

    vxlan_subnet : String
        (Optional) VXLAN IP subnet in CIDR for compute gateway

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to VMC Cloud Console.
        The certificate can be retrieved from browser.

    """

    sddc_name = name
    sddc_list = __salt__["vmc_sddc.list"](
        hostname=hostname,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        org_id=org_id,
        verify_ssl=verify_ssl,
        cert=cert,
    )

    if "error" in sddc_list:
        return vmc_state._create_state_response(
            name=name,
            comment="Failed to get SDDCs for given org : {}".format(sddc_list["error"]),
            result=False,
        )

    for sddc in sddc_list:
        if sddc["name"] == sddc_name:
            return vmc_state._create_state_response(
                name=name, comment="SDDC is already present", result=True
            )

    if __opts__.get("test"):
        log.info("vmc_sddc present is called with test option")
        return vmc_state._create_state_response(
            name=name,
            comment="SDDC {} would have been created".format(sddc_name),
        )

    created_sddc = __salt__["vmc_sddc.create"](
        hostname=hostname,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        org_id=org_id,
        sddc_name=sddc_name,
        num_hosts=num_hosts,
        provider=provider,
        region=region,
        account_link_config=account_link_config,
        account_link_sddc_config=account_link_sddc_config,
        deployment_type=deployment_type,
        host_instance_type=host_instance_type,
        msft_license_config=msft_license_config,
        sddc_id=sddc_id,
        sddc_template_id=sddc_template_id,
        sddc_type=sddc_type,
        size=size,
        skip_creating_vxlan=skip_creating_vxlan,
        sso_domain=sso_domain,
        storage_capacity=storage_capacity,
        vpc_cidr=vpc_cidr,
        vxlan_subnet=vxlan_subnet,
        validate_only=validate_only,
        verify_ssl=verify_ssl,
        cert=cert,
    )

    if "error" in created_sddc:
        return vmc_state._create_state_response(
            name=name, comment="Failed to add SDDC : {}".format(created_sddc["error"]), result=False
        )

    return vmc_state._create_state_response(
        name=name,
        comment="Created SDDC {}".format(sddc_name),
        new_state=created_sddc,
        result=True,
    )


def absent(
    name,
    hostname,
    refresh_key,
    authorization_host,
    org_id,
    force_delete=False,
    retain_configuration=False,
    template_name=None,
    verify_ssl=True,
    cert=None,
):

    """
    Ensure a given SDDC does not exist for the given organization.

    name
        Indicates the SDDC ID, UUID identifying the SDDC.

    hostname
        The host name of VMC.

    refresh_key
        API Token of the user which is used to get the Access Token required for VMC operations.

    authorization_host
        Hostname of the VMC cloud console.

    org_id
        The ID of organization to which the SDDC belongs to.

    force_delete: Boolean
        (Optional) If ``True``, will delete forcefully.
        Beware: do not use the force flag if there is a chance an active provisioning or deleting task is running
        against this SDDC. This option is restricted.

    retain_configuration: Boolean
        (Optional) If ``True``, the SDDC's configuration is retained as a template for later use.
        This flag is applicable only to SDDCs in ACTIVE state.

    template_name: String
        (Optional) Only applicable when retainConfiguration is also set to ``True``. When set, this value will be used as the name of the SDDC configuration template generated.

    verify_ssl
        (Optional) Option to enable/disable SSL verification. Enabled by default.
        If set to False, the certificate validation is skipped.

    cert
        (Optional) Path to the SSL client certificate file to connect to VMC.
        The certificate can be retrieved from browser.

    """

    sddc_id = name
    log.info("Checking if SDDC with ID %s is present", sddc_id)
    vmc_sddc = __salt__["vmc_sddc.get_by_id"](
        hostname=hostname,
        refresh_key=refresh_key,
        authorization_host=authorization_host,
        org_id=org_id,
        sddc_id=sddc_id,
        verify_ssl=verify_ssl,
        cert=cert,
    )

    if "error" in vmc_sddc:
        if "Cannot find sddc" in vmc_sddc["error"]:
            vmc_sddc = None
        else:
            return vmc_state._create_state_response(
                name=name, comment=vmc_sddc["error"], result=False
            )

    if vmc_sddc.get("sddc_state") in ("DEPLOYING", "DELETED", "DELETION_IN_PROGRESS"):
        vmc_sddc = None

    if __opts__.get("test"):
        log.info("vmc_sddc.absent is called with test option")
        if vmc_sddc:
            return vmc_state._create_state_response(
                name=name,
                comment="State absent will delete SDDC with ID {}".format(sddc_id),
            )
        else:
            return vmc_state._create_state_response(
                name=name,
                comment="State absent will do nothing as no SDDC found with ID {} or deletion is already in progress or SDDC is still deploying".format(
                    sddc_id
                ),
            )

    if vmc_sddc:
        log.info("SDDC found with ID %s", sddc_id)
        deleted_vmc_sddc = __salt__["vmc_sddc.delete"](
            hostname=hostname,
            refresh_key=refresh_key,
            authorization_host=authorization_host,
            org_id=org_id,
            sddc_id=sddc_id,
            force_delete=force_delete,
            retain_configuration=retain_configuration,
            template_name=template_name,
            verify_ssl=verify_ssl,
            cert=cert,
        )

        if "error" in deleted_vmc_sddc:
            return vmc_state._create_state_response(
                name=name, comment=deleted_vmc_sddc["error"], result=False
            )

        return vmc_state._create_state_response(
            name=name,
            comment="Deleted SDDC {}".format(sddc_id),
            old_state=vmc_sddc,
            result=True,
        )
    else:
        log.info(
            "No SDDC found with ID %s or deletion is already in progress or SDDC is still deploying.",
            sddc_id,
        )
        return vmc_state._create_state_response(
            name=name,
            comment="No SDDC found with ID {} or deletion is already in progress or SDDC is still deploying".format(
                sddc_id
            ),
            result=True,
        )
